@@ -1,31 +1,37 @@
-"""Check that your requirements.txt is up to date with the most recent package
-versions
+"""Check that your requirements.txt is up to date with the most recent package...
+
+versions.
 """
 from __future__ import annotations
-from sys import stdout
 
 import argparse
 import typing
+from pathlib import Path
+from sys import exit as sysexit
+from sys import stdout
+
+import requests
 import requirements
 from requirements.requirement import Requirement
-import requests
-lazyPrint = None
+
 try:
 	from metprint import LAZY_PRINT, LogType
-	lazyPrint = LAZY_PRINT
 except ModuleNotFoundError:
-	pass
+	LAZY_PRINT = None
 
 stdout.reconfigure(encoding="utf-8")
 
+
 class UpdateCompatible(typing.TypedDict):
-	""" UpdateCompatible type """
+	"""UpdateCompatible type."""
+
 	ver: str
 	compatible: bool
 
 
 class Dependency(typing.TypedDict):
-	""" Dependency type """
+	"""Dependency type."""
+
 	name: str
 	specs: tuple[str]
 	ver: str
@@ -33,7 +39,8 @@ class Dependency(typing.TypedDict):
 
 
 def semver(version: str) -> list[str]:
-	"""Convert a semver/ pythonver string to a list in the form major, minor,
+	"""Convert a semver/ python-ver string to a list in the form major, minor...
+
 	patch ...
 
 	Args:
@@ -46,7 +53,7 @@ def semver(version: str) -> list[str]:
 
 
 def semPad(ver: list[str], length: int) -> list[str]:
-	"""Pad a semver list to the required size. e.g. ["1", "0"] to ["1", "0", "0"]
+	"""Pad a semver list to the required size. e.g. ["1", "0"] to ["1", "0", "0"].
 
 	Args:
 		ver (list[str]): the semver representation
@@ -62,7 +69,7 @@ def semPad(ver: list[str], length: int) -> list[str]:
 
 
 def partCmp(verA: str, verB: str) -> int:
-	"""Compare parts of a semver
+	"""Compare parts of a semver.
 
 	Args:
 		verA (str): lhs part to compare
@@ -79,7 +86,7 @@ def partCmp(verA: str, verB: str) -> int:
 
 
 def _doSemCmp(semA: list[str], semB: list[str], sign: str) -> bool:
-	"""compare two semvers of equal length. e.g. 1.1.1 and 2.2.2
+	"""Compare two semvers of equal length. e.g. 1.1.1 and 2.2.2.
 
 	Args:
 		semA (list[str]): lhs to compare
@@ -112,40 +119,48 @@ def _doSemCmp(semA: list[str], semB: list[str], sign: str) -> bool:
 	# Greater than or equal. e.g. 1.1.2 >= 1.1.1
 	if sign == ">=":
 		for index, _elem in enumerate(semA):
-			if partCmp(semA[index], semB[index]) < 0:
+			cmp = partCmp(semA[index], semB[index])
+			if cmp > 0:
+				return True
+			if cmp < 0:
 				return False
 		return True
 	# Less than or equal. e.g. 1.1.1 <= 1.1.2
 	if sign == "<=":
 		for index, _elem in enumerate(semA):
-			if partCmp(semA[index], semB[index]) > 0:
+			cmp = partCmp(semA[index], semB[index])
+			if cmp < 0:
+				return True
+			if cmp > 0:
 				return False
 		return True
 	# Greater than. e.g. 1.1.2 > 1.1.1
 	if sign == ">":
 		for index, _elem in enumerate(semA[:-1]):
-			if partCmp(semA[index], semB[index]) < 0:
+			cmp = partCmp(semA[index], semB[index])
+			if cmp > 0:
+				return True
+			if cmp < 0:
 				return False
-		if partCmp(semA[-1], semB[-1]) != 1:
-			return False
-		return True
+		return False
 	# Less than. e.g. 1.1.1 < 1.1.2
 	if sign == "<":
-		for index, _elem in enumerate(semA[:-1]):
-			if partCmp(semA[index], semB[index]) > 0:
+		for index, _elem in enumerate(semA):
+			cmp = partCmp(semA[index], semB[index])
+			if cmp < 0:
+				return True
+			if cmp > 0:
 				return False
-		if partCmp(semA[-1], semB[-1]) != -1:
-			return False
-		return True
+		return False
 	raise ValueError
 
 
 def semCmp(versionA: str, versionB: str, sign: str) -> bool:
-	"""compare two semvers of any length. e.g. 1.1 and 2.2.2
+	"""Compare two semvers of any length. e.g. 1.1 and 2.2.2.
 
 	Args:
-		semA (list[str]): lhs to compare
-		semB (list[str]): rhs to compare
+		versionA (list[str]): lhs to compare
+		versionB (list[str]): rhs to compare
 		sign (str): string sign. one of ==, ~=, <=, >=, <, >
 
 	Raises:
@@ -161,8 +176,9 @@ def semCmp(versionA: str, versionB: str, sign: str) -> bool:
 
 
 def updateCompatible(req: Requirement) -> UpdateCompatible:
-	"""Check if the most recent version of a python requirement is compatible
-	with the current version
+	"""Check if the most recent version of a python requirement is compatible...
+
+	with the current version.
 
 	Args:
 		req (Requirement): the requirement object as parsed by requirements_parser
@@ -172,9 +188,9 @@ def updateCompatible(req: Requirement) -> UpdateCompatible:
 		is our requirement from requirements.txt or similar compatible
 		with the new version per the version specifier (compatible)
 	"""
-	url = "https://pypi.org/pypi/" + req.name + "/json" # type: ignore
-	request = requests.get(url) # type: ignore
-	updateVer = request.json()["info"]["version"] # type: ignore
+	url = f"https://pypi.org/pypi/{req.name}/json"
+	request = requests.get(url)
+	updateVer = request.json()["info"]["version"]
 	for spec in req.specs:
 		if not semCmp(updateVer, spec[1], spec[0]):
 			return {"ver": updateVer, "compatible": False}
@@ -182,9 +198,10 @@ def updateCompatible(req: Requirement) -> UpdateCompatible:
 
 
 def checkRequirements(requirementsFile: str) -> list[Dependency]:
-	"""Check that your requirements.txt is up to date with the most recent package
+	"""Check that your requirements.txt is up to date with the most recent package...
+
 	versions. Put in a function so dependants can use this function rather than
-	reimplement it themselves
+	reimplement it themselves.
 
 	Args:
 		requirementsFile (str): file path to the requirements file
@@ -195,27 +212,44 @@ def checkRequirements(requirementsFile: str) -> list[Dependency]:
 		(is our version compatible with ver)
 	"""
 	reqsDict = []
-	with open(requirementsFile, 'r') as requirementsTxt:
-		for req in requirements.parse(requirementsTxt): # type: ignore
-			reqsDict.append({"name": req.name, "specs": req.specs, **updateCompatible(req)}) # type: ignore
+	for req in requirements.parse(Path(requirementsFile).read_text(encoding="utf-8")):  # type: ignore
+		reqsDict.append(
+			{"name": req.name, "specs": req.specs, **updateCompatible(req)}
+		)  # type: ignore
 	return reqsDict
 
 
 def cli():
-	""" cli entry point """
+	"""CLI entry point."""
 	parser = argparse.ArgumentParser(description=__doc__)
-	parser.add_argument("--requirements-file", "-r", help="requirements file")
+	# yapf: disable
+	parser.add_argument("--requirements-file", "-r",
+	help="requirements file")
+	parser.add_argument("--zero", "-0",
+	help="Return non zero exit code if an incompatible license is found", action="store_true")
+	# yapf: enable
 	args = parser.parse_args()
-	reqsDict = checkRequirements(args.requirements_file
-	if args.requirements_file else "requirements.txt")
+	reqsDict = checkRequirements(
+		args.requirements_file if args.requirements_file else "requirements.txt"
+	)
 	if len(reqsDict) == 0:
-		_ = (print("/  WARN: No requirements") if lazyPrint is None else lazyPrint(
-		"No requirements", LogType.WARNING))
+		_ = (
+			print("/  WARN: No requirements")
+			if LAZY_PRINT is None
+			else LAZY_PRINT("No requirements", LogType.WARNING)
+		)
+	incompat = False
 	for req in reqsDict:
 		name = req["name"]
 		if req["compatible"]:
-			_ = (print("+    OK: " + name) if lazyPrint is None else lazyPrint(
-			name, LogType.SUCCESS))
+			_ = (
+				print(f"+    OK: {name}")
+				if LAZY_PRINT is None
+				else LAZY_PRINT(name, LogType.SUCCESS)
+			)
 		else:
-			_ = (print("+ ERROR: " + name) if lazyPrint is None else lazyPrint(
-			name, LogType.ERROR))
+			_ = print(f"+ ERROR: {name}") if LAZY_PRINT is None else LAZY_PRINT(name, LogType.ERROR)
+			incompat = True
+	if incompat and args.zero:
+		sysexit(1)
+	sysexit(0)
